@@ -13,18 +13,23 @@ import org.junit.runners.Suite;
  */
 @RunWith(Suite.class)
 @Suite.SuiteClasses({
-    Jdbc3CallableStatementTest.class,
-    GeneratedKeysTest.class,
     CompositeQueryParseTest.class,
-    SqlCommandParseTest.class,
-    Jdbc3SavepointTest.class,
-    TypesTest.class,
-    ResultSetTest.class,
-    ParameterMetaDataTest.class,
-    Jdbc3BlobTest.class,
+    CompositeTest.class,
     DatabaseMetaDataTest.class,
+    EscapeSyntaxCallModeCallTest.class,
+    EscapeSyntaxCallModeCallIfNoReturnTest.class,
+    EscapeSyntaxCallModeSelectTest.class,
+    GeneratedKeysTest.class,
+    Jdbc3BlobTest.class,
+    Jdbc3CallableStatementTest.class,
+    Jdbc3SavepointTest.class,
+    ParameterMetaDataTest.class,
+    ResultSetTest.class,
     SendRecvBufferSizeTest.class,
-    StringTypeParameterTest.class})
+    SqlCommandParseTest.class,
+    StringTypeParameterTest.class,
+    TypesTest.class,
+})
 public class Jdbc3TestSuite {
 
 }
